rm(list = ls())  
options(stringsAsFactors = F)
ExpMatrix = data.table::fread( "matrix of derivation cohort.txt", header = T, check.names = F,data.table = F)
ExpMatrix[1:4, 1:4]
batchType=c(rep("GSE75214",97),rep("GSE112366",167))
modType=c(rep("CD",75),rep("control",22),rep("CD",141),rep("control",26))
library(reshape2)
ExpMatrix_L1 <- melt(ExpMatrix)
colnames(ExpMatrix_L1) <- c('probe', 'sample', 'value')
ExpMatrix_L1$batch <- rep(batchType, each = nrow(ExpMatrix)) 
head(ExpMatrix_L1)
library(ggplot2)
tiff(file="test2.tiff",
     width = 50,            
     height =60,            
     units ="cm",
     compression="lzw",
     bg="white",
     res=300)
ggplot(ExpMatrix_L1, aes(x=sample, y=value)) + 
  geom_boxplot(aes(fill=batch, color=NULL), width=0.5) + 
  #stat_summary(fun.y = "mean", geom = "point", shape = 1, size = 1.5) + 
  theme_set(theme_set(theme_bw(base_size=8))) + 
  theme(text = element_text(face = 'bold'), 
        axis.text.x = element_blank(), 
        axis.title = element_blank(), 
        legend.key.size = unit(3, 'cm'),
        legend.title = element_text(size = 30), 
        legend.text = element_text(size = 30))
dev.off()
library(ggfortify)
ExpMatrix = read.table( "merge-matrix.txt", header = T, check.names = F, row.names = 1)
pca1 <- as.data.frame(t(ExpMatrix))
pca1 <- cbind(pca1, batchType)
pca1[1:4, 1:4]
pdf(file = "PCA-beforebatch.pdf")
autoplot(prcomp(pca1[ ,1:(ncol(pca1)-1)]), data = pca1, colour = 'batchType')+
  theme_set(theme_set(theme_bw(base_size=8)))
dev.off()


library(sva)
rt=read.table("matrix of derivation cohort.txt",sep="\t",header=T,check.names=F)
rt=as.matrix(rt)
rownames(rt)=rt[,1]
exp=rt[,2:ncol(rt)]
dimnames=list(rownames(exp),colnames(exp))
data=matrix(as.numeric(as.matrix(exp)),nrow=nrow(exp),dimnames=dimnames)
mod = model.matrix(~as.factor(modType))
outTab=ComBat(data, batchType, mod, par.prior=TRUE)
outTab=rbind(geneNames=colnames(outTab),outTab)
write.table(outTab,file="normalize-matrix.txt",sep="\t",quote=F,col.names=F)
ExpMatrix = read.table( "normalize-matrix.txt", header = T, check.names = F)
ExpMatrix[1:4, 1:4]
batchType=c(rep("GSE75214",97),rep("GSE112366",167))
modType=c(rep("CD",75),rep("control",22),rep("CD",141),rep("control",26))
library(reshape2)
ExpMatrix_L1 <- melt(ExpMatrix)
colnames(ExpMatrix_L1) <- c('probe', 'sample', 'value')
ExpMatrix_L1$batch <- rep(batchType, each = nrow(ExpMatrix)) 
head(ExpMatrix_L1)

library(ggplot2)
tiff(file="afterbatch.tiff",
     width = 100,            
     height =60,            
     units ="cm",
     compression="lzw",
     bg="white",
     res=300)
ggplot(ExpMatrix_L1, aes(x=sample, y=value,fill = batch)) + 
  geom_boxplot() + 
  stat_summary(fun.y = "mean", geom = "point", shape = 1, size = 1.5) + 
  theme_set(theme_set(theme_bw(base_size=8))) + 
  theme(text = element_text(face = 'bold'), 
        axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1), 
        axis.title = element_blank(), legend.key.size = unit(3, 'cm'),
        legend.title = element_text(size = 30), legend.text = element_text(size = 30))
dev.off()
library(ggfortify)
ExpMatrix = read.table( "normalize-matrix.txt", header = T, check.names = F, row.names = 1)
pca1 <- as.data.frame(t(ExpMatrix))
pca1 <- cbind(pca1, batchType)
pca1[1:4, 1:4]
pdf(file = "PCA-afterbatch.pdf")
autoplot(prcomp(pca1[ ,1:(ncol(pca1)-1)]), data = pca1, colour = 'batchType')+
  theme_set(theme_set(theme_bw(base_size=8)))
dev.off()


rm(list = ls())  
options(stringsAsFactors = F)
library(limma)
logFoldChange=1
adjustP=0.05
rt=read.table("normalize-matrix.txt",sep="\t",header=T,check.names=F)
group = read.table("merge-group.txt",sep="\t",header=T,check.names=F)
group_list = group$group

rt=as.matrix(rt)
rownames(rt)=rt[,1]
exp=rt[,2:ncol(rt)]
dimnames=list(rownames(exp),colnames(exp))
rt=matrix(as.numeric(as.matrix(exp)),nrow=nrow(exp),dimnames=dimnames)
#differential
modType=group_list
design <- model.matrix(~0+factor(modType))
colnames(design) <- c("CD","control")
fit <- lmFit(rt,design)
cont.matrix<-makeContrasts(CD-control,levels=design)
fit2 <- contrasts.fit(fit, cont.matrix)
fit2 <- eBayes(fit2)
allDiff=topTable(fit2,adjust='fdr',number=200000)
write.table(allDiff,file="alldiff.xls",sep="\t",quote=F,row.names=T)
#write table
diffSig <- allDiff[with(allDiff, (abs(logFC)>logFoldChange & adj.P.Val < adjustP )), ]
write.table(diffSig,file="diff-sig.xls",sep="\t",quote=F,row.names=T)
diffUp <- allDiff[with(allDiff, (logFC>logFoldChange & adj.P.Val < adjustP )), ]
write.table(diffUp,file="diff-up.xls",sep="\t",quote=F,row.names=T)
diffDown <- allDiff[with(allDiff, (logFC<(-logFoldChange) & adj.P.Val < adjustP )), ]
write.table(diffDown,file="diff-down.xls",sep="\t",quote=F,row.names=T)
diffSig <- cbind(id = row.names(diffSig),diffSig)
symbol <- diffSig[,1:2]
write.table(symbol,file="symbol.txt",sep="\t",quote=F,row.names=F)
#write expression level of diff gene
hmExp=rt[row.names(diffSig),]
diffExp=rbind(id=colnames(hmExp),hmExp)
write.table(diffExp,file="diffExp.txt",sep="\t",quote=F,col.names=F)


rm(list = ls())  
library("org.Hs.eg.db")
rt=read.table("symbol.txt",sep="\t",check.names=F,header=T)
genes=as.vector(rt[,1])
entrezIDs <- mget(genes, org.Hs.egSYMBOL2EG, ifnotfound=NA)
entrezIDs <- as.character(entrezIDs)
out=cbind(rt,entrezID=entrezIDs)
write.table(out,file="id.txt",sep="\t",quote=F,row.names=F)
#ClusterprofileGO
library("clusterProfiler")
library("enrichplot")
library("ggplot2")
rt=read.table("id.txt",sep="\t",header=T,check.names=F)
rt=rt[is.na(rt[,"entrezID"])==F,]
geneFC=rt$logFC
gene=rt$entrezID
names(geneFC)=gene
kk <- enrichGO(gene = gene,
               OrgDb = org.Hs.eg.db, 
               pvalueCutoff =0.05, 
               qvalueCutoff = 0.05,
               ont="all",
               readable =T)
write.table(kk,file="GO.txt",sep="\t",quote=F,row.names = F)
pdf(file="bubble-go.pdf",width = 10,height = 8)
dotplot(kk,showCategory = 10,split="ONTOLOGY") + facet_grid(ONTOLOGY~., scale='free')
dev.off()

kk <- enrichKEGG(gene = gene, organism = "hsa", pvalueCutoff =0.05, qvalueCutoff =0.05)
write.table(kk,file="KEGG.txt",sep="\t",quote=F,row.names = F)
pdf(file="bubble-kegg.pdf",width = 10,height = 7)
dotplot(kk, showCategory = 30)
dev.off()


rm(list = ls()) 
dat <- read.table("normalize-matrix.txt", sep = "\t", header = T, row.names = 1 )
group <- read.table("merge-group.txt", sep = "\t", header = T)
group_list <- group$group
dat[1:4,1:4]  
table(group_list)
library(GSVA)
library(limma)
library(GSEABase)
gmtFile="c2.cp.kegg.v6.2.symbols.gmt"
exp=as.matrix(dat)
dimnames=list(rownames(exp),colnames(exp))
mat=matrix(as.numeric(as.matrix(exp)),nrow=nrow(exp),dimnames=dimnames)
mat=avereps(mat)
mat=normalizeBetweenArrays(mat)
c3gsc2=getGmt( gmtFile, 
               collectionType=BroadCollection(category="c2"), 
               geneIdType=SymbolIdentifier())
gsvaOut=gsva(mat, 
             c3gsc2, 
             min.sz=10, 
             max.sz=500, 
             verbose=TRUE,
             parallel.sz=1)
gsvaOut=rbind(id=colnames(gsvaOut),gsvaOut)
write.table(gsvaOut,file="gsvaOut-kegg.txt",sep="\t",quote=F,col.names=F)
#find different pathway
rt=read.table("gsvaOut-kegg.txt",sep="\t",header=T,check.names=F,row.names=1)
logFCcutoff=0.2
adjPvalueCutoff=0.05
design <- model.matrix(~0+factor(group_list))
colnames(design)=levels(factor(group_list))
head(design)
exprSet=rt
rownames(design)=colnames(exprSet)
design
contrast.matrix<-makeContrasts("CD-control",
                             levels = design)
contrast.matrix 
fit <- lmFit(exprSet,design)
fit2 <- contrasts.fit(fit, contrast.matrix) 
fit2 <- eBayes(fit2)  
all=topTable(fit2, coef= 1, number=Inf,adjust.method="holm")
all=rbind(id=colnames(all),all)
write.table(all,file="allgsva-kegg.txt",sep="\t",quote=F,col.names=F)
diff <- topTable(fit2, coef= 1, number=Inf,
                 p.value=adjPvalueCutoff, adjust="holm", lfc=logFCcutoff)
diffName=row.names(diff)
diff=rbind(id=colnames(diff),diff)
write.table(diff,file="diffgsva-kegg.xls",sep="\t",quote=F,col.names=F)
hmExp=rt[diffName,]
hmExp=rbind(id=colnames(hmExp),hmExp)
write.table(hmExp,file="heatmapgsva-kegg.txt",sep="\t",quote=F,col.names=F)
Geo=c(rep("GSE75214",97),rep("GSE112366",167))
Type=c(rep("CD",75),rep("control",22),rep("CD",141),rep("control",26))    
names(Geo)=colnames(rt)
ann=cbind(Geo,Type)
ann=as.data.frame(ann)
pdf(file="heatmap-GSVA-kegg.pdf", width = 65, height = 50 )
pheatmap(rt, annotation=ann, #according to the number of DEG
         color = colorRampPalette(c("green", "black", "red"))(50),
         cluster_cols =F,
         #         scale="row",
         fontsize_row=40,
         fontsize_col=30,
         cellwidth =10,
         cellheight = 60,
         fontsize = 30, 
         border_color = NA
)
dev.off()

rm(list=ls())
library(ggplot2)
library(ggpubr)
library(gridExtra) 
library(dplyr)
library(tidyr)
library(RColorBrewer)
rt1 <- read.table("CIBERSORTx_CD.txt",sep="\t",header=T,check.names=F)
rt2 <- read.table("CIBERSORTx_NC.txt",sep="\t",header=T,check.names=F)
rt <- rbind(rt1,rt2)
rt <- rt[rt$`P-value`<0.05,]
rt <- rt[,1:23]
colnames(rt)[1] <- 'id'
write.table(rt,file = 'CIBERSORTx_All.txt', quote = F, sep = '\t', row.names = F)
group = read.table("merge-group.txt",sep="\t",header=T,check.names=F)
group$group <- ifelse(group$group=='control','NC','CD')
rt <- merge(group, rt, by='id')
rt <- rt[order(rt$group),]
rownames(rt) <- rt$id
rt <- rt[,2:ncol(rt)]
rt$group <- factor(rt$group,levels=c("NC","CD"))
data <- gather(rt,value = 'expression', key = "cell", -c('group')) # transform to long data
#set comparison
compaired<-list(c("NC","CD"))
#draw figure
pdf(file = 'immunecells_boxplot.pdf',width = 26,height = 13)
ggplot(data, aes(x=group,y=expression)) + 
  stat_boxplot(geom = "errorbar",width=0.5)+
  geom_boxplot(fill="white",color="black",outlier.alpha = 0,alpha=0)+
  geom_jitter(aes(color=group,shape=group),stat = "identity",width = 0.15,alpha=1)+ 
  scale_color_lancet()+
  facet_wrap(.~cell,scales = "free",ncol = 11)+
  guides(shape=F)+
  theme_bw()+
  theme(strip.text = element_text(face = "bold",hjust = 0.5,size = 9),
        strip.background = element_rect(fill = "LightGray",colour = "black", size = 0),
        axis.text.x = element_blank(),
        axis.ticks.x = element_blank(),
        axis.title.x = element_blank(),
        axis.text.y = element_text(face = "bold",size = 15),
        axis.title.y = element_text(face = "bold",size = 20),
        legend.background = element_blank(),
        legend.key.size = unit(1,'cm'),
        #legend.key.width = unit(0,'cm'),
        #legend.key.height = unit(0,'cm'),
        legend.text = element_text(face = "bold",size = 20),
        legend.title = element_text(face = "bold",size = 20))+
  labs(y="Absolute score",color="Group")+
  geom_signif(comparisons = compaired,
              step_increase = 0.1,
              map_signif_level = T,
              test = wilcox.test,
              col="black",
              textsize = 5)
dev.off()

rm(list = ls())
input="CIBERSORTx_All.txt"
outpdf="barplot.pdf"
data <- read.table(input,header=T,sep="\t",check.names=F)
rt <- gather(data,key = cell, value = fraction,-'id') #transform to long data
pdf(outpdf,height=5,width=15)
ggplot(rt,aes(id,weight = fraction, fill=cell))+
  geom_bar(color = 'black',position = 'fill')+
  ylab('Fraction')+ 
  xlab('')+
  scale_y_continuous(expand = c(0,0))+ 
  theme_classic()+
  theme(axis.text.x = element_blank(),axis.title.x = element_text(colour = "red", size = rel(1.5)))
dev.off()

rm(list=ls())
rt=read.table("CIBERSORTx_All.txt",sep="\t",header=T,check.names=F)
group = read.table("merge-group.txt",sep="\t",header=T,check.names=F)
rt <- merge(group, rt, by='id')
rt <- rt[order(rt$group),]
save(rt,file = 'CIBERSORTx+group.Rdata')
rownames(rt) <- rt$id
rt <- as.data.frame(t(rt[,3:ncol(rt)]))

library(pheatmap)
Type=c(rep("CD",216),rep("NC",48))    
names(Type)=colnames(rt)
Type=as.data.frame(Type)
ann_colors = list(
  Type = c(NC = "#1B9E77", CD = "#D95F02")
)
pdf("heatmap-immune.pdf",height=5,width=15)
pheatmap(rt, 
         annotation=Type, 
         color = colorRampPalette(c('blue', "white", "red"))(50),
         cluster_cols =F,
         fontsize = 8,
         fontsize_row=7,
         fontsize_col=5,
         border_color = NA,
         cellwidth = 3,
         annotation_colors = ann_colors,
         show_colnames = F,
         scale = 'row')
dev.off()

library(corrplot)
library(ggcorrplot)
rm(list = ls())       
rt1=read.table("CIBERSORTx_CD.txt",sep="\t",header=T,check.names=F)
rt1 <- rt1[,1:23]
rt1 <- rt1[,-c(2,6,13,19,22)] #delet the immune cell didn't alter between two group
colnames(rt1)[1] <- 'id'


rt2=read.table("hubExp.txt",sep="\t",header=T,row.names=1,check.names=F)
rt2 <- as.data.frame(t(rt2))
rt2 <- rt2[rt1[,1],]
rt2 <- cbind(id=rownames(rt2),rt2)

rt3=read.table("heatmapgsva-kegg.txt",sep="\t",header=T,row.names=1,check.names=F)
rt3 <- as.data.frame(t(rt3))
rt3 <- rt3[rt1[,1],]
rt3 <- cbind(id=rownames(rt3),rt3)
pathway <- c("KEGG_PROTEASOME","KEGG_PATHOGENIC_ESCHERICHIA_COLI_INFECTION","KEGG_NOD_LIKE_RECEPTOR_SIGNALING_PATHWAY",
             "KEGG_DRUG_METABOLISM_CYTOCHROME_P450","KEGG_SYSTEMIC_LUPUS_ERYTHEMATOSUS")
rt3 <- rt3[,c('id',pathway)]

rt <- merge(rt1,rt2,by='id')
rt <- merge(rt,rt3,by='id')
write.table(rt,'corheatmap.txt', quote = F, sep = '\t', row.names = F)
rownames(rt) <- rt$id
rt <- rt[,2:ncol(rt)]


save(rt1,rt2,rt3,rt,file = 'immunecell_pathway_hubgene.Rdata')

corr <- round(cor(rt,method = 'spearman'), 2)
corr_out <- cbind(id=rownames(corr),corr)
write.table(corr_out,'correlation_coeficient.xls', sep = '\t', row.names = F)

p.mat <- cor_pmat(rt)
p.mat_out <- cbind(id=rownames(p.mat),p.mat)
write.table(p.mat_out,'correlation_pvalue.xls', sep = '\t', row.names = F)

pdf("corHeatmap-keggimunehub.pdf",height=20,width=20)
corrplot(corr, 
         method = "color", 
         type= "full",
         col = colorRampPalette(c("#084594", "white", "#ef3b2c"))(50),
         tl.col = 'black',
         tl.cex = 1.2,
         p.mat=p.mat,
         sig.level = 0.05,
         insig = 'blank',
         addCoef.col = NULL,
         addgrid.col = NA,
         cl.pos = 'b',
         cl.cex = 1.2,
         order = 'hclust'
)
dev.off()

rm(list = ls())
options(stringsAsFactors = F)
corFilter=0.3           #coef of correlation
pvalueFilter=0.05        #p-value

#input the gene expression data
#data from the corheatmap,row is gene and col is sample!!!
load('immunecell_pathway_hubgene.Rdata')

#extract and tranform the data,row name is gene, pathway and immune cell
immuneCell <- as.data.frame(t(rt[,1:17]))
gene <- as.data.frame(t(rt[,c(18:32)]))
pathway <- as.data.frame(t(rt[,c(33:37)]))

outTab1=data.frame()
for(i in row.names(gene)){
  for(j in row.names(immuneCell)){
    x=as.numeric(gene[i,])
    y=as.numeric(immuneCell[j,])
    corT=cor.test(x,y,method = 'spearman')
    cor=corT$estimate
    pvalue=corT$p.value
    if((cor>corFilter) & (pvalue<pvalueFilter)){
      outTab1=rbind(outTab1,cbind(gene=i,immuneCell=j,cor,pvalue,Regulation="postive"))
    }
    if((cor< -corFilter) & (pvalue<pvalueFilter)){
      outTab1=rbind(outTab1,cbind(gene=i,immuneCell=j,cor,pvalue,Regulation="negative"))
    }
  }
}

outTab2=data.frame()
for(i in row.names(gene)){
  for(j in row.names(pathway)){
    x=as.numeric(gene[i,])
    y=as.numeric(pathway[j,])
    corT=cor.test(x,y,method = 'spearman')
    cor=corT$estimate
    pvalue=corT$p.value
    if((cor>corFilter) & (pvalue<pvalueFilter)){
      outTab2=rbind(outTab2,cbind(gene=i,pathway=j,cor,pvalue,Regulation="postive"))
    }
    if((cor< -corFilter) & (pvalue<pvalueFilter)){
      outTab2=rbind(outTab2,cbind(gene=i,pathway=j,cor,pvalue,Regulation="negative"))
    }
  }
}

outTab3=data.frame()
for(i in row.names(immuneCell)){
  for(j in row.names(pathway)){
    x=as.numeric(immuneCell[i,])
    y=as.numeric(pathway[j,])
    corT=cor.test(x,y,method = 'spearman')
    cor=corT$estimate
    pvalue=corT$p.value
    if((cor>corFilter) & (pvalue<pvalueFilter)){
      outTab3=rbind(outTab3,cbind(immuneCell=i,pathway=j,cor,pvalue,Regulation="postive"))
    }
    if((cor< -corFilter) & (pvalue<pvalueFilter)){
      outTab3=rbind(outTab3,cbind(immuneCell=i,pathway=j,cor,pvalue,Regulation="negative"))
    }
  }
}
colnames(outTab1)[2] <- 'target'
colnames(outTab2)[2] <- 'target'
colnames(outTab3)[2] <- 'target'
save(outTab1, outTab2, outTab3, file = 'outTabdata.Rdata')

outTab <- rbind(outTab1, outTab2)
outTab4 <- outTab[outTab$gene==c('CXCL8'),]
outTab5 <- outTab[outTab$gene==c('IL1B'),]
outTab <- rbind(outTab4, outTab5)

write.table(file="corResult_gene&pathway&immunecell.txt",outTab,sep="\t",quote=F,row.names=F) 

#output the node label of network
geneLabel=cbind(rownames(gene),"Immunegene")
pathwaypLabel=cbind(rownames(pathway),"pathway")
immunecellLabel=cbind(rownames(immuneCell),"ImmuneCell")
nodeLabel=rbind(c("ID","type"),geneLabel,pathwaypLabel,immunecellLabel)
write.table(nodeLabel,file="nodeType.txt",sep="\t",quote=F,col.names=F,row.names=F)

rm(list = ls())
load('immunecell_pathway_hubgene.Rdata')
load('outTabdata.Rdata')
rt1 <- rt[,c(2,3,5,7,11,12,16,17,18,19)] #extract the immune cell have correlation coef >0.3 with IL1B and CXCL8
rt2 <- rt[,c(2,3,5,7,11,12,16,17,35)] #extract the hub gene and immune cell with correlation coef >0.3

cor1=cor(rt1,method = 'spearman')
cor2=cor(rt2,method = 'spearman')

library(circlize)
library(corrplot)
options(stringsAsFactors=F)

# draw circos for hub gene and pathway
if(T){
  #set figure color
  pdf("circos_hubgene_pathway.pdf",width = 8)
  col = c(rgb(1,0,0,seq(1,0,length=32)),rgb(0,1,0,seq(0,1,length=32)))
  cor1[cor1==1]=0
  par(mar=c(1,2,1,4),cex=0.7)
  c1 = ifelse(c(cor1)>=0,rgb(1,0,0,abs(cor1)),rgb(0,1,0,abs(cor1)))
  col1 = matrix(c1,nc=ncol(rt1))
  
  circos.par(gap.degree =c(3,rep(2, nrow(cor1)-1)),start.degree = 180)
  chordDiagram(cor1,grid.col=rainbow(ncol(rt1)),transparency = 0.5,col=col1,symmetric = T)
  par(xpd=T)
  colorlegend(col, vertical = T,labels=c(1,0,-1),xlim=c(1.1,1.3),ylim=c(-0.4,0.4))       
  dev.off()
  circos.clear()
}

# draw circos for hub gene and immune cell
if(T){
  #set figure color
  pdf("circos_hubgene_immunecell.pdf",width = 8)
  col = c(rgb(1,0,0,seq(1,0,length=32)),rgb(0,1,0,seq(0,1,length=32)))
  cor2[cor2==1]=0
  par(mar=c(1,2,1,4),cex=0.7)
  c1 = ifelse(c(cor2)>=0,rgb(1,0,0,abs(cor2)),rgb(0,1,0,abs(cor2)))
  col1 = matrix(c1,nc=ncol(rt2))
  
  circos.par(gap.degree =c(3,rep(2, nrow(cor2)-1)),start.degree = 180)
  chordDiagram(cor2,grid.col=rainbow(ncol(rt2)),transparency = 0.5,col=col1,symmetric = T)
  par(xpd=T)
  colorlegend(col, vertical = T,labels=c(1,0,-1),xlim=c(1.1,1.3),ylim=c(-0.4,0.4))       
  dev.off()
  circos.clear()
}


rm(list = ls())
library("glmnet")
library("survival")
library(ggplot2)
library(broom)
library(ggrepel)
library(dplyr)
rt1=read.table("CIBERSORTx_All.txt",sep="\t",header=T,check.names=F)
rt1 <- rt1[,1:23]
rt1 <- rt1[,-c(2,6,13,19,22)] #delet the immune cell didn't alter between two group
colnames(rt1)[1] <- 'id'
rt2=read.table("hubExp.txt",sep="\t",header=T,row.names=1,check.names=F)
rt2 <- as.data.frame(t(rt2))
rt2 <- cbind(id=rownames(rt2),rt2)
rt3=read.table("heatmapgsva-kegg.txt",sep="\t",header=T,row.names=1,check.names=F)
rt3 <- as.data.frame(t(rt3))
rt3 <- cbind(id=rownames(rt3),rt3)
pathway <- c("KEGG_PROTEASOME","KEGG_PATHOGENIC_ESCHERICHIA_COLI_INFECTION","KEGG_NOD_LIKE_RECEPTOR_SIGNALING_PATHWAY",
             "KEGG_DRUG_METABOLISM_CYTOCHROME_P450","KEGG_SYSTEMIC_LUPUS_ERYTHEMATOSUS")
rt3 <- rt3[,c('id',pathway)]
rt <- merge(rt1,rt2,by='id')
rt <- merge(rt,rt3,by='id')
write.table(rt,'allsample_cell_hubgene_pathway.txt',quote = F, sep = '\t', row.names = F)
rownames(rt) <- rt$id
rt <- rt[,2:ncol(rt)]
rt <- rt[,c(2,3,5,7,11,12,16,17,18,19,35)]
rt <- cbind(id=rownames(rt), rt)
group <- read.table("merge-group.txt",sep="\t",header=T,check.names=F)
rt    <- merge(group, rt, by='id')
rt    <- rt[order(rt$group),]
class(rt$group)
rt$group <- ifelse(rt$group=='CD',1,0)
rownames(rt) <- rt$id
rt <- rt[,2:ncol(rt)]
x=as.matrix(rt[,-1])
y=rt[,1]
set.seed(3)#3
fit <- glmnet(x, y, family = "binomial", alpha = 1)
cvfit <- cv.glmnet(x, y, family="binomial")
if(T){
  tidied_cv = tidy(cvfit)
  
  pdf("cvfit.pdf",width = 8)
  p <- ggplot(tidied_cv, aes(lambda, estimate)) +
    geom_point(color = "red",size=2) +
    geom_errorbar(aes(ymin = conf.low, ymax = conf.high), alpha = .3) +
    scale_x_log10()+
    geom_vline(xintercept = cvfit$lambda.min, lty=2,color = "red",size=1) +
    geom_vline(xintercept = cvfit$lambda.1se, lty=2) +
    ylab("Binomial Deviance")+
    annotate("text",x=cvfit$lambda.min/5,size=5,
             y =max(tidied_cv$estimate),
             label =paste0("lambda.min = ",
                           round(cvfit$lambda.min,3),
                           "\n",
                           "number = ",tidied_cv$nzero[tidied_cv$lambda==cvfit$lambda.min]))+
    ggtitle("")+
    theme_bw()+
    theme(axis.title = element_text(size = 15, colour = 'black'),
          axis.text = element_text(size = 15, colour = 'black'))
  print(p)
  dev.off()
}
coef <- coef(fit, s = cvfit$lambda.min)
index <- which(coef != 0)
actCoef <- coef[index]
lassoGene=row.names(coef)[index]
if(T){
  tidy_covariates = tidy(cvfit$glmnet.fit)
  index <- tidy_covariates$term %in% lassoGene
  data_point <- tidy_covariates %>% 
    filter(index) %>% 
    arrange(lambda) %>% 
    distinct(term,.keep_all = T)
  
  pdf("lambda.pdf",width = 8)
  g <- ggplot(tidy_covariates)+
    geom_line(data = subset(tidy_covariates,index), aes(x=lambda, y=estimate, color=as.factor(term)),size=1) +
    geom_line(data = subset(tidy_covariates,!index), aes(x=lambda, y=estimate, color=as.factor(term)),size=0.8,alpha=0.3) +
    guides(color=FALSE) +
    geom_vline(xintercept = cvfit$lambda.min, lty=2,color='red') +
    #scale_color_manual(values=col_sam)+
    scale_x_log10()+
    ylab("Coefficients")+
    geom_point(data =data_point,aes(lambda,estimate,color=as.factor(term)),size=3)+
    geom_text_repel(data =data_point,aes(lambda,estimate,label=term),size=3)+
    theme_bw()+
    theme(axis.title = element_text(size = 15, colour = 'black'),
          axis.text = element_text(size = 15, colour = 'black'))
  print(g)
  dev.off()
}
lassoGene
lassoGene=c("group",lassoGene[2:7])
lassoSigExp=rt[,lassoGene]
lassoSigExp=cbind(id=row.names(lassoSigExp),lassoSigExp)
write.table(lassoSigExp,file="lassoSigExp.txt",sep="\t",row.names=F,quote=F)

rm(list=ls())
options(stringsAsFactors = F)
if(T){
  library(ggplot2)
  library(survminer)
  library(survival)
  library(rms)
  library(forestplot)
  library(pROC)
  library(regplot)
  library(sm)
  library(vioplot)
  library(beanplot)
  library(epiDisplay)
  
}
#input from origninal
load('hubgene_pathway_immunece.Rdata')
rt <- rt[,c('CXCL8',"IL1B","KEGG_NOD_LIKE_RECEPTOR_SIGNALING_PATHWAY", "Macrophages M1", "Neutrophils" )]
rt <- cbind(id=rownames(rt), rt)
group <- read.table("merge-group.txt",sep="\t",header=T,check.names=F)
rt    <- merge(group, rt, by='id')
class(rt$group)
rt    <- rt[order(rt$group),]
rt$group <- ifelse(rt$group=='CD',1,0)
rownames(rt) <- rt$id
rt <- rt[,2:ncol(rt)]
colnames(rt)[5] <- 'M1'

#input from lasso regression
rt <- read.table('lassoSigExp.txt',header = T, sep = '\t', check.names = F, row.names = 1)
class(rt$group)

#logistica regression
logisticCD  <- glm (group ~., data = rt, family = binomial)
summary(logisticCD)
logistic.display(logisticCD)
residualPlots(logisticCD)
#ROC curve
pdf(file = 'ROC_CD_diagnostic_test.pdf')
pre1 <- predict(logisticCD,type='response')
modelroc1 <- roc(rt$group,pre1)
plot(modelroc1, print.auc=TRUE, auc.polygon=TRUE, grid=c(0.1, 0.2),
     grid.col=c("green", "red"), max.auc.polygon=TRUE,
     auc.polygon.col="skyblue", print.thres=TRUE)
dev.off()
save(logisticCD,file = "logisticmodel.Rdata")
colnames(rt)[1:5] <- c('CD','Plasmacell','CD8','Treg','M1')
dd<-datadist(rt)
options(datadist="dd")
options(na.action="na.delete")
mycol<-c("#A6CEE3","#1F78B4","red","#2166AC")
names(mycol) = c("dencol","boxcocl","obscol","spkcol")
mycol<- as.list(mycol)
pbcglm1 <- glm(CD~., family = "binomial", data=rt )
#pdf('nomogram_CD_NC.pdf',width = 20, height = 8)
regplot(pbcglm1, 
        observation=rt[5,],
        odds=F, 
        interval="confidence",
        droplines = F,
        subticks = T,
        colors = mycol,
        title = "")
dev.off()



##validation cohort
rm(list = ls())  
ExpMatrix <- data.table::fread( "matrix of validation cohort.txt", header = T, check.names = F,data.table = F)
batchType=c(rep("GSE3365",101),rep("GSE10616",48),rep("GSE16879",49),rep("GSE102133",77))
modType=c(rep("CD",59),rep("control",42),rep("CD",32),rep("control",16),
          rep("CD",37),rep("control",12),rep("CD",65),rep("control",12))
ExpMatrix_L1 <- melt(ExpMatrix)
colnames(ExpMatrix_L1) <- c('probe', 'sample', 'value')
ExpMatrix_L1$batch <- rep(batchType, each = nrow(ExpMatrix)) 
head(ExpMatrix_L1)

library(ggplot2)
tiff(file="beforebatch.tiff",
     width = 100,            
     height =60,            
     units ="cm",
     compression="lzw",
     bg="white",
     res=300)
ggplot(ExpMatrix_L1, aes(x=sample, y=value,fill = batch)) + 
  geom_boxplot() + 
  stat_summary(fun.y = "mean", geom = "point", shape = 1, size = 1.5) + 
  theme_set(theme_set(theme_bw(base_size=8))) + 
  theme(text = element_text(face = 'bold'), 
        axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1), 
        axis.title = element_blank(), legend.key.size = unit(3, 'cm'),
        legend.title = element_text(size = 30), legend.text = element_text(size = 30))
dev.off()
pdf(file="beforebatch.pdf",
    width = 100,            
    height =60            
)
ggplot(ExpMatrix_L1, aes(x=sample, y=value)) + 
  geom_boxplot(aes(fill=batch, color=NULL), width=0.5) + 
  #stat_summary(fun.y = "mean", geom = "point", shape = 1, size = 1.5) + 
  theme_set(theme_set(theme_bw(base_size=8))) + 
  theme(text = element_text(face = 'bold'), 
        axis.text.x = element_blank(), 
        axis.title = element_blank(), 
        legend.key.size = unit(3, 'cm'),
        legend.title = element_text(size = 30), 
        legend.text = element_text(size = 30))
dev.off()
## PCA
library(ggfortify)
ExpMatrix = read.table( "merge_matrix.txt", header = T, check.names = F, row.names = 1)
pca1 <- as.data.frame(t(ExpMatrix))
pca1 <- cbind(pca1, batchType)
pca1[1:4, 1:4]
dat.pca <- PCA(pca1[,-ncol(pca1)], graph = FALSE)
pdf(file = "PCA-beforebatch.pdf")
fviz_pca_ind(dat.pca,
             geom.ind = "point", # show points only (nbut not "text")
             col.ind = pca1$batchType, # color by groups
             # palette = c("#00AFBB", "#E7B800"),
             addEllipses = TRUE, # Concentration ellipses
             legend.title = "Groups"
)
dev.off()
library(sva)
rt=read.table("merge_matrix.txt",sep="\t",header=T,check.names=F)
rt=as.matrix(rt)
rownames(rt)=rt[,1]
exp=rt[,2:ncol(rt)]
dimnames=list(rownames(exp),colnames(exp))
data=matrix(as.numeric(as.matrix(exp)),nrow=nrow(exp),dimnames=dimnames)
mod = model.matrix(~as.factor(modType))
outTab=ComBat(data, batchType, mod, par.prior=TRUE)
outTab=rbind(geneNames=colnames(outTab),outTab)
write.table(outTab,file="normalize-matrix.txt",sep="\t",quote=F,col.names=F)

rm(list = ls())  
options(stringsAsFactors = F)
ExpMatrix = read.table( "normalize-matrix.txt", header = T, check.names = F)
ExpMatrix[1:4, 1:4]

batchType=c(rep("GSE3365",101),rep("GSE10616",48),rep("GSE16879",49),rep("GSE102133",77))
modType=c(rep("CD",59),rep("control",42),rep("CD",32),rep("control",16),
          rep("CD",37),rep("control",12),rep("CD",65),rep("control",12))
ExpMatrix_L1 <- melt(ExpMatrix)
colnames(ExpMatrix_L1) <- c('probe', 'sample', 'value')
ExpMatrix_L1$batch <- rep(batchType, each = nrow(ExpMatrix)) 
head(ExpMatrix_L1)

tiff(file="afterbatch.tiff",
     width = 100,            
     height =60,            
     units ="cm",
     compression="lzw",
     bg="white",
     res=300)
ggplot(ExpMatrix_L1, aes(x=sample, y=value,fill = batch)) + 
  geom_boxplot() + 
  stat_summary(fun.y = "mean", geom = "point", shape = 1, size = 1.5) + 
  theme_set(theme_set(theme_bw(base_size=8))) + 
  theme(text = element_text(face = 'bold'), 
        axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1), 
        axis.title = element_blank(), legend.key.size = unit(3, 'cm'),
        legend.title = element_text(size = 30), legend.text = element_text(size = 30))
dev.off()

pdf(file="afterbatch.pdf",
    width = 100,            
    height =60            
)
ggplot(ExpMatrix_L1, aes(x=sample, y=value,fill = batch)) + 
  geom_boxplot() + 
  stat_summary(fun.y = "mean", geom = "point", shape = 1, size = 1.5) + 
  theme_set(theme_set(theme_bw(base_size=8))) + 
  theme(text = element_text(face = 'bold'), 
        axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1), 
        axis.title = element_blank(), legend.key.size = unit(3, 'cm'),
        legend.title = element_text(size = 30), legend.text = element_text(size = 30))
dev.off()
## PCA1
ExpMatrix = read.table( "normalize-matrix.txt", header = T, check.names = F, row.names = 1)
pca1 <- as.data.frame(t(ExpMatrix))
pca1 <- cbind(pca1, batchType)
pca1[1:4, 1:4]

dat.pca <- PCA(pca1[,-ncol(pca1)], graph = FALSE)
pdf(file = "PCA-afterbatch.pdf")
fviz_pca_ind(dat.pca,
             geom.ind = "point", # show points only (nbut not "text")
             col.ind = pca1$batchType, # color by groups
             # palette = c("#00AFBB", "#E7B800"),
             addEllipses = TRUE, # Concentration ellipses
             legend.title = "Groups"
)
dev.off()

rm(list = ls())  
options(stringsAsFactors = F)
logFoldChange=1
adjustP=0.05
rt=read.table("normalize-matrix.txt",sep="\t",header=T,check.names=F)
group_list = c(rep("CD",59),rep("control",42),rep("CD",32),rep("control",16),
               rep("CD",37),rep("control",12),rep("CD",65),rep("control",12))
rt=as.matrix(rt)
rownames(rt)=rt[,1]
exp=rt[,2:ncol(rt)]
dimnames=list(rownames(exp),colnames(exp))
rt=matrix(as.numeric(as.matrix(exp)),nrow=nrow(exp),dimnames=dimnames)
#differential
modType=group_list
design <- model.matrix(~0+factor(modType))
colnames(design) <- c("CD","control")
fit <- lmFit(rt,design)
cont.matrix<-makeContrasts(CD-control,levels=design)
fit2 <- contrasts.fit(fit, cont.matrix)
fit2 <- eBayes(fit2)
allDiff=topTable(fit2,coef = 1,adjust='fdr',number=200000)
write.table(allDiff,file="alldiff.xls",sep="\t",quote=F,row.names=T)
#write table
diffSig <- allDiff[with(allDiff, (abs(logFC)>logFoldChange & adj.P.Val < adjustP )), ]
write.table(diffSig,file="diff-sig.xls",sep="\t",quote=F,row.names=T)
diffUp <- allDiff[with(allDiff, (logFC>logFoldChange & adj.P.Val < adjustP )), ]
write.table(diffUp,file="diff-up.xls",sep="\t",quote=F,row.names=T)
diffDown <- allDiff[with(allDiff, (logFC<(-logFoldChange) & adj.P.Val < adjustP )), ]
write.table(diffDown,file="diff-down.xls",sep="\t",quote=F,row.names=T)
diffSig <- cbind(id = row.names(diffSig),diffSig)
symbol <- diffSig[,1:2]
write.table(symbol,file="symbol.txt",sep="\t",quote=F,row.names=F)
#write expression level of diff gene
hmExp=rt[row.names(diffSig),]
diffExp=rbind(id=colnames(hmExp),hmExp)
write.table(diffExp,file="diffExp.txt",sep="\t",quote=F,col.names=F)
rt=read.table("normalize-matrix.txt",sep="\t",header=T,check.names=F)
rt=as.matrix(rt)
rownames(rt)=rt[,1]
exp=rt[,2:ncol(rt)]
dimnames=list(rownames(exp),colnames(exp))
rt=matrix(as.numeric(as.matrix(exp)),nrow=nrow(exp),dimnames=dimnames)
#rt=log2(rt+1)
#rt[rt>15]=15

library(pheatmap)
Geo=c(rep("GSE3365",101),rep("GSE10616",48),rep("GSE16879",49),rep("GSE102133",77))
Type=c(rep("CD",59),rep("control",42),rep("CD",32),rep("control",16),
       rep("CD",37),rep("control",12),rep("CD",65),rep("control",12))    
names(Geo)=colnames(rt)
ann=cbind(Geo,Type)
ann=as.data.frame(ann)

tiff(file="heatmap-all.tiff",
     width = 100,            
     height =50,            
     units ="cm",
     compression="lzw",
     bg="white",
     res=300)
pheatmap(rt[1:200, ], annotation=ann, 
         color = colorRampPalette(c("blue", "white", "red"))(50),
         cluster_cols =F,
         #         scale="row",
         fontsize_row=5,
         fontsize_col=5,
         cellwidth =5,
         cellheight =5, fontsize = 15
)
dev.off()
##for DEG
rt=read.table("diffExp.txt",sep="\t",header=T,check.names=F)
rt=as.matrix(rt)
rownames(rt)=rt[,1]
exp=rt[,2:ncol(rt)]
dimnames=list(rownames(exp),colnames(exp))
rt=matrix(as.numeric(as.matrix(exp)),nrow=nrow(exp),dimnames=dimnames)
library(pheatmap)
Geo=c(rep("GSE75214",78),rep("GSE112366",167))
Type=c(rep("CD",67),rep("control",11),rep("CD",141),rep("control",26))  
ann=cbind(Geo,Type)
ann=as.data.frame(ann)
pdf(file="heatmap-diff.pdf", width = 45, height = 15 )
pheatmap(rt, annotation=ann, #according to the number of DEG
         color = colorRampPalette(c("blue", "white", "red"))(50),
         cluster_cols =F,
         scale="row",
         fontsize_row=10,
         fontsize_col=10,
         cellwidth =10,
         cellheight = 10,
         fontsize = 20, 
         border_color = NA,
         show_colnames = F
)
dev.off()

rm(list=ls())

rt1 <- read.table("CIBERSORTx_CD.txt",sep="\t",header=T,check.names=F)
rt2 <- read.table("CIBERSORTx_NC.txt",sep="\t",header=T,check.names=F)
rt <- rbind(rt1,rt2)
rt <- rt[rt$`P-value`<0.05,]
rt <- rt[,1:23]
colnames(rt)[1] <- 'id'
write.table(rt,file = 'CIBERSORTx_All.txt', quote = F, sep = '\t', row.names = F)
group = c(rep("CD",193),rep("NC",82))
rt <- cbind(group=group, rt)
rownames(rt) <- rt$id
rt <- rt[,-2]
rt$group <- factor(rt$group,levels=c("NC","CD"))
data <- gather(rt,value = 'expression', key = "cell", -c('group')) # transform to long data

#set comparison
compaired<-list(c("NC","CD"))
#draw figure
pdf(file = 'immunecells_boxplot.pdf',width = 26,height = 13)
ggplot(data, aes(x=group,y=expression)) + 
  stat_boxplot(geom = "errorbar",width=0.5)+
  geom_boxplot(fill="white",color="black",outlier.alpha = 0,alpha=0)+
  geom_jitter(aes(color=group,shape=group),stat = "identity",width = 0.15,alpha=1)+ 
  scale_color_lancet()+
  facet_wrap(.~cell,scales = "free",ncol = 11)+
  guides(shape=F)+
  theme_bw()+
  theme(strip.text = element_text(face = "bold",hjust = 0.5,size = 9),
        strip.background = element_rect(fill = "LightGray",colour = "black", size = 0),
        axis.text.x = element_blank(),
        axis.ticks.x = element_blank(),
        axis.title.x = element_blank(),
        axis.text.y = element_text(face = "bold",size = 15),
        axis.title.y = element_text(face = "bold",size = 20),
        legend.background = element_blank(),
        legend.key.size = unit(1,'cm'),
        #legend.key.width = unit(0,'cm'),
        #legend.key.height = unit(0,'cm'),
        legend.text = element_text(face = "bold",size = 20),
        legend.title = element_text(face = "bold",size = 20))+
  labs(y="Absolute score",color="Group")+
  geom_signif(comparisons = compaired,
              step_increase = 0.1,
              map_signif_level = T,
              test = wilcox.test,
              col="black",
              textsize = 5)
dev.off()
library(RColorBrewer)
library(dplyr)
library(tidyr)
rm(list = ls())
input="CIBERSORTx_All.txt"
outpdf="barplot.pdf"

data <- read.table(input,header=T,sep="\t",check.names=F)
rt <- gather(data,key = cell, value = fraction,-'id') #transform to long data

pdf(outpdf,height=5,width=15)
ggplot(rt,aes(id,weight = fraction, fill=cell))+
  geom_bar(color = 'black',position = 'fill')+
  ylab('Fraction')+ 
  xlab('')+
  scale_y_continuous(expand = c(0,0))+ 
  theme_classic()+
  theme(axis.text.x = element_blank(),axis.title.x = element_text(colour = "red", size = rel(1.5)))
dev.off()
rm(list=ls())
rt=read.table("CIBERSORTx_All.txt",sep="\t",header=T,check.names=F)
group = c(rep("CD",193),rep("NC",82))
rt <- cbind(group=group, rt)
save(rt,file = 'CIBERSORTx+group.Rdata')
rownames(rt) <- rt$id
rt <- as.data.frame(t(rt[,-c(1,2)]))

Type=c(rep("CD",193),rep("NC",82))    
names(Type)=colnames(rt)
Type=as.data.frame(Type)
ann_colors = list(
  Type = c(NC = "#1B9E77", CD = "#D95F02")
)

pdf("heatmap-immune.pdf",height=5,width=15)
pheatmap(rt, 
         annotation=Type, 
         color = colorRampPalette(c('blue', "white", "red"))(50),
         cluster_cols =F,
         fontsize = 8,
         fontsize_row=7,
         fontsize_col=5,
         border_color = NA,
         cellwidth = 3,
         annotation_colors = ann_colors,
         show_colnames = F,
         scale = 'row')
dev.off()
rm(list=ls())

#input from origninal
load("CIBERSORTx+group.Rdata")
group <- rt[,1:2]
rt1 <- data.table::fread('normalize-matrix.txt',header = T, sep = '\t', check.names = F, data.table = F)
rownames(rt1) <- rt1[,1]
rt1 <- rt1[,-1]
rt1 <- as.data.frame(t(rt1[c("IL1B","CXCL8"),]))
rt1 <- cbind(id=rownames(rt1),rt1)
rt2 <- data.table::fread('CIBERSORTx_All.txt',header = T, sep = '\t', check.names = F, data.table = F)
colnames(rt2)
rt2 <- rt2[,c("id","Plasma cells","Macrophages M1","T cells CD8","T cells regulatory (Tregs)")]
rt <- merge(rt1,rt2,by="id")
rt <- merge(group,rt,by="id")
rownames(rt) <- rt$id
rt <- rt[,-1]
rt$group <- ifelse(rt$group=='CD',1,0)
#ROC curve
load("logisticmodel.Rdata")
pre1 <- predict(logisticCD,type='response',newdata = rt)
modelroc1 <- roc(rt$group,pre1)
pdf(file = 'ROC_CD_diagnostic.pdf')
plot(modelroc1, print.auc=TRUE, auc.polygon=TRUE, grid=c(0.1, 0.2),
     grid.col=c("green", "red"), max.auc.polygon=TRUE,
     auc.polygon.col="skyblue", print.thres=TRUE)
dev.off()
Cindex <- rcorrcens(rt$group~pre1)
Cindex
se <- 0.041/2 #SD from cindex
cil <- 0.884-1.96*se
ciu <- 0.884+1.96*se
#calibration curve
f1 <- lrm(group ~., data = rt, x=T, y=T) # calibrate must be from lrm
residualPlots(f1)
autoplot(f1)
Cindex <- rcorrcens(rt$CD~predict(f1))
Cindex
se <- 0.041/2 #SD from cindex
cil <- 0.915-1.96*se
ciu <- 0.915+1.96*se
validate(f1,method="boot",B=3840,dxy=T)
cal<-calibrate(f1,method="boot",B=1000)
pdf('Calibrate_Nomogram_CD_NC.pdf',width =10, height = 8)
plot(cal,scat1d.opts=list(nhistSpike=100,side=1,frac=0.08,lwd=1,nint=50),xlab='Predict Probability')
dev.off()
summary(cal)
